#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from gettext import gettext as _

from gi import require_version
require_version("Gtk", "3.0")

from gi.repository import Gtk
from gi.repository import Gdk

from .configuration import *
from .utils import *

class ApplicationWindow(Gtk.ApplicationWindow):
    def __init__(self, app):
        Gtk.ApplicationWindow.__init__(self,
                                       title=APP_NAME,
                                       application=app)

        """
        Build Settings
        """
        self.app = app
        self.settings = Gtk.Settings.get_default()
        self.set_position(Gtk.WindowPosition.CENTER)

        # Set window size & position
        self.set_default_size(320, 480)
        #print (self.get_position())
        #print (self.get_size())

        # Prefer dark theme
        self.settings.set_property("gtk-application-prefer-dark-theme", APP_PREFER_DARK_THEME)

        # Prefer dark theme on custom themes
        if is_linux() and APP_PREFER_DARK_THEME:
            theme = self.settings.get_property("gtk-theme-name")
            os.environ["GTK_THEME"] = theme + ":dark"

    def build(self):
        self.set_icon_name(APP_ICON_NAME)

        headerbar = Gtk.HeaderBar()
        headerbar.set_title(APP_NAME)
        headerbar.set_show_close_button(True)
        self.set_titlebar(headerbar)

class AboutWindow(Gtk.AboutDialog):
    def __init__(self, main_window = None):
        Gtk.AboutDialog.__init__(self)

        self.main_window = main_window
        self.connect("response", self.dialog_response)

        """
        Build Settings
        """
        self.set_modal(True)
        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        self.set_transient_for(self.main_window)
        self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)

    def build(self):
        self.set_program_name(APP_NAME)
        self.set_website(APP_URL)
        self.set_version(APP_VERSION)
        self.set_comments(APP_DESCRIPTION)
        self.set_authors(APP_AUTHORS)
        self.set_translator_credits(' '.join(APP_TRANSLATORS))
        self.set_documenters(APP_DOCUMENTERS)
        self.set_license_type(APP_LICENSE)
        self.set_copyright(APP_COMMENTS)
        self.set_icon_name(APP_ICON_NAME)
        self.set_logo_icon_name(APP_LOGO_NAME)

    def dialog_response(self, dialog, response):
        dialog.destroy()

class PrefernecesWindow(Gtk.Window):
    def __init__(self, main_window = None):
        Gtk.Window.__init__(self)

        self.main_window = main_window
        self.connect("key-press-event", self._keypress)

        """
        Build Settings
        """
        self.set_title(_("Preferneces"))
        self.set_default_size(480, 400)
        self.set_modal(True)
        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        self.set_transient_for(self.main_window)
        self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)

        """
        Build HeaderBar
        """
        self.headerbar = Gtk.HeaderBar()
        self.headerbar.set_title(_("Preferneces"))
        self.headerbar.set_show_close_button(True)
        self.set_titlebar(self.headerbar)

    def _keypress(self, widget, event):
        if event.keyval == Gdk.KEY_Escape:
            self.destroy()

    def build(self):
        """
        Build Content
        """
        self.layout = Gtk.Box()
        self.layout.set_orientation(Gtk.Orientation.VERTICAL)
        self.layout.set_border_width(0)
        self.add(self.layout)

        self.pages = {}
        self.listboxes = {}
        self.preferences = None
        self.listbox = None
        self.notebook = None
        """
        self.add_preference2("Title of option", "switch", self.on_switch_activated, None)
        self.add_preference2("Title of option 2", "switch", self.on_switch_activated, None)
        self.add_preference2("Title of option 3", "switch", self.on_switch_activated, None)
        """
        self.add_preference2("Title of option", "switch", self.on_switch_activated, None, "General")
        self.add_preference2("Title of option 2", "switch", self.on_switch_activated, None, "General")
        self.add_preference2("Title of option 3", "switch", self.on_switch_activated, None, "General")

        return



        self.notebook = Gtk.Notebook()
        self.notebook.set_tab_pos(Gtk.PositionType.TOP) # LEFT, RIGHT, TOP, BOTTOM
        self.layout.pack_end(self.notebook, True, True, 0)

        # Tab
        self.pageGeneral = Gtk.Box()
        self.pageGeneral.set_border_width(10)

        # At tab label
        self.notebook.append_page(self.pageGeneral, Gtk.Label(_("General")))

        ## Listbox
        self.listbox = Gtk.ListBox()
        self.listbox.set_selection_mode(Gtk.SelectionMode.NONE)
        self.pageGeneral.pack_start(self.listbox, True, True, 0)

        ## Row
        ### Create list row
        row = Gtk.ListBoxRow()

        ### Create list row content box
        hbox = Gtk.Box()
        hbox.set_orientation(Gtk.Orientation.HORIZONTAL)
        hbox.set_property("homogeneous", True)

        #### Create list row content box label
        label = Gtk.Label("Title of option")
        label.set_halign(Gtk.Align.START)
        label.set_valign(Gtk.Align.CENTER)
        hbox.pack_start(label, True, True, 0)

        #### Create list row content box switch
        switch = Gtk.Switch()
        switch.set_halign(Gtk.Align.END)
        switch.set_valign(Gtk.Align.CENTER)
        switch.set_active(True)
        switch.connect("notify::active", self.on_switch_activated)
        hbox.pack_start(switch, False, True, 0)

        ### Row content add to listbox row
        row.add(hbox)
        ### Row add to listbox
        self.listbox.add(row)




        """
        self.add_preference("Title of option", "switch", None)
            self.add_listbox()
                self.add_preference()

        """

        self.notebook = None
        #self.add_preference2("Title of option", "switch", None)
        self.add_preference2("Title of option", "switch", None, "General")


        self.notebook = None
        general = self.add_tab("General")
        general_lsb = self.add_listbox(general)
        self.add_preference(general_lsb, "Title of option", "switch", None)
        self.add_preference(general_lsb, "Title of option 2", "switch", None)
        self.add_preference(general_lsb, "demo", "switch", None)
        general2 = self.add_tab("General 2")
        general2_lsb = self.add_listbox(general2)
        self.add_preference(general2_lsb, "G2: Title of option", "switch", None)
        self.add_preference(general2_lsb, "G2: Title of option 2", "switch", None)
        self.add_preference(general2_lsb, "G2: demo", "switch", None)










        """
        Build Search
        """
        self.search_icon = Gtk.Image()
        self.search_icon.set_from_icon_name("edit-find-symbolic", Gtk.IconSize.BUTTON)
        self.search_button = Gtk.ToggleButton()
        self.search_button.add(self.search_icon)
        self.search_button.connect("toggled", self._on_transition)
        self.headerbar.pack_start(self.search_button)

        self.search_revealer = Gtk.Revealer()
        self.search_entry = Gtk.SearchEntry()
        self.search_entry.set_placeholder_text(_("Search Preferences"))
        self.search_entry.set_property("margin_left", 10)
        self.search_entry.set_property("margin_right", 10)
        self.search_entry.set_property("margin_top", 5)
        self.search_entry.set_property("margin_bottom", 5)
        self.search_entry.connect("search-changed", self._on_search)
        self.search_revealer.add(self.search_entry)

        self.listbox.set_filter_func(self._preference_filter, None)

        self.layout.pack_start(self.search_revealer, False, False, 0)

    def add_preference2(self, title, type, callback, data = None, page = None):
        ## Row
        ### Create list row
        row = Gtk.ListBoxRow()

        ### Create list row content box
        hbox = Gtk.Box()
        hbox.set_orientation(Gtk.Orientation.HORIZONTAL)
        hbox.set_property("homogeneous", True)

        #### Create list row content box label
        label = Gtk.Label("Title of option")
        label.set_halign(Gtk.Align.START)
        label.set_valign(Gtk.Align.CENTER)
        hbox.pack_start(label, True, True, 0)

        #### Create list row content box switch
        switch = Gtk.Switch()
        switch.set_halign(Gtk.Align.END)
        switch.set_valign(Gtk.Align.CENTER)
        switch.set_active(True)
        switch.connect("notify::active", callback)
        hbox.pack_start(switch, False, True, 0)

        ### Row content add to listbox row
        row.add(hbox)
        ### Row add to listbox


        # page set?
        if page != None:
            # Yes, append to that
            if self.notebook == None:
                # Create notebook
                self.notebook = Gtk.Notebook()
                self.notebook.set_tab_pos(Gtk.PositionType.TOP) # LEFT, RIGHT, TOP, BOTTOM
                self.notebook.set_scrollable(True)
                # Add notebook to layout
                self.layout.pack_end(self.notebook, True, True, 0)

            # Create tab & page if it does not exists
            if page not in self.pages:
                # Add containers for each page
                self.pages[page] = Gtk.Box()
                self.pages[page].set_border_width(10)
                # Add listboxs for each page
                self.listboxes[page] = Gtk.ListBox()
                self.listboxes[page].set_selection_mode(Gtk.SelectionMode.NONE)
                # Add listbox to container
                self.pages[page].pack_start(self.listboxes[page], True, True, 0)
                # Add container to notebook
                self.notebook.append_page(self.pages[page], Gtk.Label(_(page)))
            # Add row to listbox
            self.listboxes[page].add(row)

        else:
            # No, append to window
            # If no container, create
            if self.preferences == None:
                # Add containers for each page
                self.preferences = Gtk.Box()
                self.preferences.set_border_width(10)
                # Add listboxs for each page
                self.listbox = Gtk.ListBox()
                self.listbox.set_selection_mode(Gtk.SelectionMode.NONE)
                # Add listbox to container
                self.preferences.pack_start(self.listbox, True, True, 0)
                # Add container to layout
                self.layout.pack_end(self.preferences, True, True, 0)
            # Add row to listbox
            self.listbox.add(row)










    def add_tab(self, label):
        if self.notebook == None:
            self.notebook = Gtk.Notebook()
            self.notebook.set_tab_pos(Gtk.PositionType.TOP) # LEFT, RIGHT, TOP, BOTTOM
            self.layout.pack_end(self.notebook, True, True, 0)
        panel = Gtk.Box()
        panel.set_border_width(10)
        self.notebook.append_page(panel, Gtk.Label(_(label)))
        return panel

    def add_listbox(self, container):
        listbox = Gtk.ListBox()
        listbox.set_selection_mode(Gtk.SelectionMode.NONE)
        container.pack_start(listbox, True, True, 0)
        return listbox

    def add_preference(self, target, title, type, data):
        ## Row
        ### Create list row
        row = Gtk.ListBoxRow()

        ### Create list row content box
        hbox = Gtk.Box()
        hbox.set_orientation(Gtk.Orientation.HORIZONTAL)
        hbox.set_property("homogeneous", True)

        #### Create list row content box label
        label = Gtk.Label(_(title))
        label.set_halign(Gtk.Align.START)
        label.set_valign(Gtk.Align.CENTER)
        hbox.pack_start(label, True, True, 0)

        #### Create list row content box switch
        if type == "switch":
            switch = Gtk.Switch()
            switch.set_halign(Gtk.Align.END)
            switch.set_valign(Gtk.Align.CENTER)
            switch.set_active(True)
            switch.connect("notify::active", self.on_switch_activated)
            hbox.pack_start(switch, False, True, 0)
        else:
            pass

        ### Row content add to listbox row
        row.add(hbox)
        ### Row add to listbox
        target.add(row)











    def _on_search(self, entry):
        self.listbox.invalidate_filter()

    def _preference_filter(self, lista, user_data):
        text = self.search_entry.get_text()
        if not text:
            return lista
        """
        lista        (Gtk.ListBox) -> .get_child
        get the rows (Gtk.ListBoxRow) -> .git_children
        get the label Gtk.Box -> [0] (a.k.a label)
        """
        lbl = lista.get_child().get_children()[0]
        if text.lower() in lbl.get_text().lower():
            return lista

    def _on_transition(self, btn):
        if self.search_revealer.get_reveal_child():
            self.search_revealer.set_reveal_child(False)
            self.search_entry.set_text("")
            btn.grab_focus()
        else:
            self.search_revealer.set_reveal_child(True)
            self.search_entry.grab_focus()

    def _on_key_press(self, widget, event):
        keyname = Gdk.keyval_name(event.keyval)
        if keyname == 'Escape':
            self.search_button.set_active(False)
        if event.state and Gdk.ModifierType.CONTROL_MASK:
            if keyname == 'f':
                self.search_button.set_active(True)

    def on_switch_activated(self, switch, gparam):
        if switch.get_active():
            state = "on"
        else:
            state = "off"

        print("Switch was turned", state)

class ShortcutsWindow(Gtk.ShortcutsWindow):
    group_titles = {}
    group_actions = {}
    groups = []

    def __init__(self, main_window = None, app = None):
        Gtk.ShortcutsWindow.__init__(self)

        self.main_window = main_window
        self.app = app

        """
        Build Settings
        """
        self.set_modal(True)
        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        self.set_transient_for(self.main_window)
        self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)

    def build(self):
        """Gathers the accelerators and populates the window."""
        section = Gtk.ShortcutsSection()
        section.show()
        for group_id in ShortcutsWindow.groups:
            group = Gtk.ShortcutsGroup(title=ShortcutsWindow.group_titles[group_id])
            group.show()
            for action, title in ShortcutsWindow.group_actions[group_id]:
                accelerators = " ".join(self.app.get_accels_for_action(action))
                short = Gtk.ShortcutsShortcut(title=title, accelerator=accelerators)
                short.show()
                group.add(short)
            section.add(group)
        # Method below must be called after the section has been populated,
        # otherwise the shortcuts won't show up in search.
        self.add(section)

    @classmethod
    def add_action(cls, action, title, group=None):
        """Adds an action to be displayed.

        Args:
            action (str): The name identifying the action, formatted like
                "prefix.name".
            title (str): The title of the action.
            group (Optional[str]): The group id registered with `register_group`
                to be used instead of the one extracted from `action`.
        """
        action_prefix = group or action.split(".")[0]
        if action_prefix not in cls.group_actions:
            cls.group_actions[action_prefix] = []
        cls.group_actions[action_prefix].append((action, title))

    @classmethod
    def register_group(cls, action_prefix, title):
        """Registers a group of shortcuts to be displayed.

        Args:
            action_prefix (str): The group id.
            title (str): The title of the group.
        """
        if action_prefix not in cls.groups:
            cls.groups.append(action_prefix)
        cls.group_titles[action_prefix] = title
